command

package
v1.0.0-pre Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 21, 2026 License: MIT Imports: 15 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

var (

	// file pull pool
	FileBufPool = sync.Pool{
		New: func() any {
			return new(bufio.Writer)
		},
	}
	// file pull synchronizer
	FileSynchronizer sync.WaitGroup
	FileErrChan      = make(chan error)
)
var (
	INTRO   = "enjoy using puzzle! :-)"
	VERSION = "unknown"
	BUILD   = "unknown"
	DESC    = "a cli tool to simplifies the generation of project."
	DATE    = "-"
)

Functions

func GetFileBuf

func GetFileBuf(w io.Writer) *bufio.Writer

func MountBuiltinInit

func MountBuiltinInit(rootCmd *cobra.Command)

func MountBuiltinNew

func MountBuiltinNew(rootCmd *cobra.Command)

func MountBuiltinVersion

func MountBuiltinVersion(rootCmd *cobra.Command)

MountVersion mount the verb `-version` to the command tree. For details, see the `command.json` file in the project structure to find the structure of the corresponding command.

func PutFileBuf

func PutFileBuf(buf *bufio.Writer)

Types

type GithubApi

type GithubApi struct{}

func (GithubApi) PathInfo

func (a GithubApi) PathInfo(format string, remote *config.GithubConfig, path string) string

func (GithubApi) RawFile

func (a GithubApi) RawFile(format string, remote *config.GithubConfig, path string) string

Jump to