Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ( // file pull pool FileBufPool = sync.Pool{ New: func() any { return new(bufio.Writer) }, } // file pull synchronizer FileSynchronizer sync.WaitGroup FileErrChan = make(chan error) )
var ( INTRO = "enjoy using puzzle! :-)" VERSION = "unknown" BUILD = "unknown" DESC = "a cli tool to simplifies the generation of project." DATE = "-" )
Functions ¶
func MountBuiltinInit ¶
func MountBuiltinNew ¶
func MountBuiltinVersion ¶
MountVersion mount the verb `-version` to the command tree. For details, see the `command.json` file in the project structure to find the structure of the corresponding command.
func PutFileBuf ¶
Types ¶
type ContentNode ¶
Click to show internal directories.
Click to hide internal directories.